/*
 * JBoss, Home of Professional Open Source
 * Copyright 2013, Red Hat, Inc., and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.weld.tests.injectionPoint.observer;

import static org.junit.Assert.assertTrue;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.enterprise.event.Event;
import jakarta.inject.Inject;
import jakarta.inject.Named;

@ApplicationScoped
@Named("dispatcher")
public class EventDispatcher {

    @Inject
    private Event<Foo> fooEvent;

    @Inject
    @Alpha
    private transient Event<Object> barEvent;

    public void fireEvents() {
        fireFoo();
        fireBar();
    }

    private void fireFoo() {
        Foo foo = new Foo();
        fooEvent.fire(foo);
        assertTrue(foo.isObserved());
    }

    private void fireBar() {
        Bar bar = new Bar();
        barEvent.select(Bravo.Literal.INSTANCE).select(Charlie.Literal.INSTANCE).select(Bar.class).fire(bar);
        assertTrue(bar.isObserved());
    }
}
